
#include <stdio.h>
#include <hip/hip_runtime.h>

#if __HIP__
__hip_pinned_shadow__
#endif
texture<float, 1, hipReadModeElementType> tex_ref_A;


/* vector Assign A to B */
__global__ void vectorAssign(float *B, int numElements)
{
    int i = blockDim.x * blockIdx.x + threadIdx.x;

    if (i < numElements)
    {
        B[i] = tex1Dfetch(tex_ref_A, i);
    }
}

/* Host main routine */
int main(void)
{
    // Print the vector length to be used, and compute its size
    int numElements = 100000;
    size_t size = numElements * sizeof(float);

    float *h_A = (float *)malloc(size);
    float *h_B = (float *)malloc(size);

    // Initialize the host input vectors
    for (int i = 0; i < numElements; ++i)
    {
        h_A[i] = rand()/(float)RAND_MAX;
    }

/*
    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_A[i]);
	}
	printf("\n");

    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_B[i]);
	}
	printf("\n");
*/

    // Allocate the device input vector A
    float *d_A = NULL;
    hipMalloc((void **)&d_A, size);
    float *d_B = NULL;
    hipMalloc((void **)&d_B, size);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_A, h_A, size, hipMemcpyHostToDevice);

	tex_ref_A.normalized = false;
	tex_ref_A.filterMode = hipFilterModePoint;
	tex_ref_A.addressMode[0] = hipAddressModeClamp;

	hipError_t err;
	size_t offset = 0;
	err = hipBindTexture(&offset, tex_ref_A, d_A, size);
	hipGetTextureAlignmentOffset(&offset, &tex_ref_A);
	if ( err != hipSuccess ) { 
		fprintf( stderr, "cannot bind A to texture: %s (%d)\n", hipGetErrorString(err), err );
		return -1;
	}

    int threadsPerBlock = 256;
    int blocksPerGrid =(numElements + threadsPerBlock - 1) / threadsPerBlock;
    hipLaunchKernelGGL((vectorAssign), dim3(blocksPerGrid), dim3(threadsPerBlock), 0, 0, d_B, numElements);
    hipMemcpy(h_B, d_B, size, hipMemcpyDeviceToHost);

	//check res
	for (int i = 0; i < numElements; ++i)
	{   
		if(fabs(h_A[i] - h_B[i]) > 1e-5)
		{   
			fprintf(stderr, "Result verification failed at element %d!\n", i); 
			exit(EXIT_FAILURE);
		}   
	}

/*
    for(int i=0; i<numElements; i++)
	{
		printf("%f\n", h_B[i]);
	}
*/

    // Free device global memory
    hipFree(d_A);
    hipFree(d_B);

    // Free host memory
    free(h_A);
    free(h_B);

    printf("Done\n");
    return 0;
}
