
#include <stdio.h>
#include "hip/hip_runtime_api.h"
#include "hip/hip_vector_types.h"
#include "rocfft.h"
#include "hipfft.h"

#define NX 4
#define NY 4
#define NZ 4
#define BATCH 1

/* Host main routine */
int main(void)
{
    hipfftHandle plan1;
    hipfftHandle plan2;

    // Print the vector length to be used, and compute its size
    int lx = NX;
    int ly = NY;
    int lz = NZ;
    int lxyz = lx*ly*lz;
    int lzc = (NZ/2+1);
    int lzr2 = lzc*2;
    int lxyzc = lx*ly*lzc;
    int lxyzr2 = lx*ly*lzr2;

    size_t sizeR2 = lxyzr2 * sizeof(hipfftReal);

    // Allocate the host
    hipfftReal *h_A = (hipfftReal *)malloc(sizeR2);
    hipfftReal *h_B = (hipfftReal *)malloc(sizeR2);
	memset(h_A, 0, sizeR2);
	memset(h_B, 0, sizeR2);

    // Initialize the host input vectors
    for(int i=0; i<lx; ++i)
    {
        for(int j=0; j<ly; ++j)
        {
            for(int k=0; k<lz; ++k)
            {
                int index=i*ly*lzr2 + j*lzr2 +k;
                h_A[index] = rand()/(float)RAND_MAX;
            }
        }
    }

    for(int i=0; i<lxyzr2; i++)
    {
        printf("%f\n", h_A[i]);
    }
    printf("\n");

    // Allocate the device
    hipfftReal *d_A = NULL;
    hipMalloc((void **)&d_A, sizeR2);

    // Copy the host input vectors A and B in host memory to the device input vectors in device memory
    hipMemcpy(d_A, h_A, sizeR2, hipMemcpyHostToDevice);

    hipfftPlan3d(&plan1, lx, ly, lz, HIPFFT_R2C);
    hipfftPlan3d(&plan2, lx, ly, lz, HIPFFT_C2R);

    hipfftExecR2C(plan1, d_A, (hipfftComplex *)d_A);
    hipfftExecC2R(plan2, (hipfftComplex *)d_A, d_A);

    hipDeviceSynchronize();

    // Copy the device result vector in device memory to the host result vector in host memory.
    hipMemcpy(h_B, d_A, sizeR2, hipMemcpyDeviceToHost);
    for(int i=0; i<lxyzr2; i++)
    {
        printf("%f\n", h_B[i]/lxyz);
    }
    printf("\n");

    hipfftDestroy(plan1);
    hipfftDestroy(plan2);
    // Free device global memory
    hipFree(d_A);

    // Free host memory
    free(h_A);
    free(h_B);

    printf("Done\n");
    return 0;
}

