#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <hip/hip_runtime.h>
#include <mpi.h>

int main(int argc, char **argv)
{
    int myid, numprocs;
	char myhostname[1024];
	char *s_buf_h, *r_buf_h;
	char *s_buf_d, *r_buf_d;

	int devsrc = atoi(argv[1]);
	int devdst = atoi(argv[2]);

    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
    MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

	gethostname(myhostname, 1024);
	//printf("Process: %d, On host: %s, devsrc: %d, devdst: %d\n", myid, myhostname, devsrc, devdst);

	//hip set device
	if(myid == 0) {
		hipSetDevice(devsrc);
	}
	else
	{
		hipSetDevice(devdst);
	}

	//init buffer
	int numElements = 10;
	size_t size = numElements * sizeof(char);

	if(myid == 0) {
		s_buf_h = (char *)malloc(size);
		memset(s_buf_h, 'a', size);
		hipError_t hiperr = hipMalloc((void**)&s_buf_d, size);
		if (hipSuccess != hiperr) {
			fprintf(stderr, "Could not allocate device memory\n");
			return 1;
		}
		hipMemcpy(s_buf_d, s_buf_h, size, hipMemcpyHostToDevice);
		//hipMemset((void *)s_buf_d, 'a', numElements);
	}
	else {
		r_buf_h = (char *)malloc(size);
		memset(r_buf_h, 'b', size);
		hipMalloc((void**)&r_buf_d, size);
		hipMemcpy(r_buf_d, r_buf_h, size, hipMemcpyHostToDevice);
		//hipMemset(r_buf_d, 'b', numElements);
	}

	//output init buffer
	if(myid == 0) {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(s_buf_h, s_buf_d, size, hipMemcpyDeviceToHost);
		printf("s_buf_h:\n");
		for(int i=0; i<numElements; ++i) {
			printf("%c ", s_buf_h[i]);
		}
		printf("\n");
	}
	else {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(r_buf_h, r_buf_d, size, hipMemcpyDeviceToHost);
		printf("r_buf_h:\n");
		for(int i=0; i<numElements; ++i) {
			printf("%c ", r_buf_h[i]);
		}
		printf("\n");
	}


	//send data
	MPI_Status status;
	int tag = 0;
	if(myid == 1) {
		MPI_Recv(r_buf_d, numElements, MPI_CHAR, 0, tag, MPI_COMM_WORLD, &status);
	}
	else if(myid == 0) {
		MPI_Send(s_buf_d, numElements, MPI_CHAR, 1, tag, MPI_COMM_WORLD);
	}
	MPI_Barrier(MPI_COMM_WORLD);

	//output buffer
	if(myid == 0) {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(s_buf_h, s_buf_d, size, hipMemcpyDeviceToHost);
		printf("s_buf_h:\n");
		for(int i=0; i<numElements; ++i) {
			printf("%c ", s_buf_h[i]);
		}
		printf("\n");
	}
	else {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(r_buf_h, r_buf_d, size, hipMemcpyDeviceToHost);
		printf("r_buf_h:\n");
		for(int i=0; i<numElements; ++i) {
			printf("%c ", r_buf_h[i]);
		}
		printf("\n");
	}


    if(myid == 0) {
		printf("Done\n");
        fflush(stdout);
    }

    MPI_Finalize();
    return 0;
}
