#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>

#include <mpi.h>
#include <hip/hip_runtime.h>

// Error handling macros
#define hipErrorCheck(a) do { \
    if (hipSuccess != (a)) { \
		fprintf(stderr, "HIP runtime error in line %d of file %s \
			: %s \n", __LINE__, __FILE__, hipGetErrorString(a) ); \
		exit(EXIT_FAILURE); \
	} \
} while(0);

void initData(float *data, int dataSize)
{
	for(int i = 0; i < dataSize; i++)
	{
		data[i] = (float)rand() / RAND_MAX;
	}
}

int main(int argc, char **argv)
{
    int myid, numprocs;
    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &myid);
    MPI_Comm_size(MPI_COMM_WORLD, &numprocs);

	char myhostname[1024];
	gethostname(myhostname, 1024);
	printf("Process: %d, On host: %s\n", myid, myhostname);

	//init buffer
	//size_t numElements = (1<<30)/4;
	size_t numElements = 10;
	size_t sizePerNode = numElements * sizeof(float);

	float *dataNode = (float *)malloc(sizePerNode);


	if(myid == 0) {
		initData(dataNode, numElements);
		for (int i = 0; i<numElements; ++i)
		{   
			printf("%f\n", dataNode[i]);
		}

		//float *deviceInputData = NULL;
		//hipMalloc((void**)s_buf_d, sizePerNode)
		//hipErrorCheck(hipMalloc((void**)s_buf_d, size));
		//sleep(30);
		//hipErrorCheck(hipMemcpy(s_buf_d, s_buf_h, size, hipMemcpyHostToDevice));
		//hipMemcpy(s_buf_d, s_buf_h, size, hipMemcpyHostToDevice);
	}

/*
	//send data
	MPI_Status status;
	int tag = 0;
	if(myid == 1) {
		MPI_Recv(r_buf_d, numElements, MPI_FLOAT, 0, tag, MPI_COMM_WORLD, &status);
	}
	else if(myid == 0) {
		MPI_Send(s_buf_d, numElements, MPI_FLOAT, 1, tag, MPI_COMM_WORLD);
	}
	MPI_Barrier(MPI_COMM_WORLD);

	//output buffer
	if(myid == 0) {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(s_buf_h, s_buf_d, size, hipMemcpyDeviceToHost);
		printf("s_buf_h:\n");
		for(int i=0; i<10; ++i) {
			printf("%f\n", s_buf_h[i]);
		}
		printf("\n");
	}
	else {
		printf("Process: %d, On host: %s\n", myid, myhostname);
		hipMemcpy(r_buf_h, r_buf_d, size, hipMemcpyDeviceToHost);
		printf("r_buf_h:\n");
		for(int i=0; i<10; ++i) {
			printf("%f\n", r_buf_h[i]);
		}
		printf("\n");
	}
*/

	MPI_Barrier(MPI_COMM_WORLD);

	free(dataNode);

    if(myid == 0) {
		printf("Done\n");
        fflush(stdout);
    }
    MPI_Finalize();
    return 0;
}
